@@ -4,7 +4,7 @@
 -- |
 -- Module      :  Data.ByteString.Lex.Integral
 -- Copyright   :  Copyright (c) 2010--2015 wren gayle romano
--- License     :  BSD3
+-- License     :  BSD2
 -- Maintainer  :  wren@community.haskell.org
 -- Stability   :  provisional
 -- Portability :  Haskell98
@@ -12,6 +12,8 @@
 -- Functions for parsing and producing 'Integral' values from\/to
 -- 'ByteString's based on the \"Char8\" encoding. That is, we assume
 -- an ASCII-compatible encoding of alphanumeric characters.
+--
+-- /Since: 0.3.0/
 ----------------------------------------------------------------
 module Data.ByteString.Lex.Integral
     (
@@ -44,6 +46,7 @@ import           Data.Bits
 import           Foreign.Ptr              (Ptr, plusPtr)
 import qualified Foreign.ForeignPtr       as FFI (withForeignPtr)
 import           Foreign.Storable         (peek, poke)
+import           Data.ByteString.Lex.Internal
 
 ----------------------------------------------------------------
 ----- General
@@ -54,7 +57,7 @@ import           Foreign.Storable         (peek, poke)
 -- provide both signed and unsigned versions of the
 -- {read,pack}{Decimal,Octal,Hex} functions...
 
-
+-- TODO: move to somewhere more general, shared by both Integral and Fractional
 -- | Adjust a reading function to recognize an optional leading
 -- sign. As with the other functions, we assume an ASCII-compatible
 -- encoding of the sign characters.
@@ -113,7 +116,7 @@ readDecimalSimple = start
 -- call 'fromIntegral' to perform the conversion at the end. However,
 -- doing this will make your code succeptible to overflow bugs if
 -- the target type is larger than @Int@.
-readDecimal :: Integral a => ByteString -> Maybe (a, ByteString)
+readDecimal :: (Integral a) => ByteString -> Maybe (a, ByteString)
 {-# SPECIALIZE readDecimal ::
     ByteString -> Maybe (Int,     ByteString),
     ByteString -> Maybe (Int8,    ByteString),
@@ -132,7 +135,7 @@ readDecimal = start
     {-# INLINE isDecimal #-}
     isDecimal w = 0x39 >= w && w >= 0x30
 
-    toDigit :: Integral a => Word8 -> a
+    toDigit :: (Integral a) => Word8 -> a
     {-# INLINE toDigit #-}
     toDigit w = fromIntegral (w - 0x30)
 
@@ -141,7 +144,7 @@ readDecimal = start
     addDigit n w = n * 10 + toDigit w
     
     -- TODO: should we explicitly drop all leading zeros before we jump into the unrolled loop?
-    start :: Integral a => ByteString -> Maybe (a, ByteString)
+    start :: (Integral a) => ByteString -> Maybe (a, ByteString)
     start xs
         | BS.null xs = Nothing
         | otherwise  =
@@ -149,7 +152,7 @@ readDecimal = start
             w | isDecimal w -> Just $ loop0 (toDigit w) (BSU.unsafeTail xs)
               | otherwise   -> Nothing
 
-    loop0 :: Integral a => a -> ByteString -> (a, ByteString)
+    loop0 :: (Integral a) => a -> ByteString -> (a, ByteString)
     loop0 m xs
         | m `seq` xs `seq` False = undefined
         | BS.null xs = (m, BS.empty)
@@ -159,7 +162,7 @@ readDecimal = start
               | otherwise   -> (m, xs)
 
     loop1, loop2, loop3, loop4, loop5, loop6, loop7, loop8
-        :: Integral a => a -> Int -> ByteString -> (a, ByteString)
+        :: (Integral a) => a -> Int -> ByteString -> (a, ByteString)
     loop1 m n xs
         | m `seq` n `seq` xs `seq` False = undefined
         | BS.null xs = (m*10 + fromIntegral n, BS.empty)
@@ -225,7 +228,9 @@ readDecimal = start
 -- the string, and returns @0@ instead of @Nothing@. This is twice
 -- as fast for 'Int64' on 32-bit systems, but has identical performance
 -- to 'readDecimal' for all other types and architectures.
-readDecimal_ :: Integral a => ByteString -> a
+--
+-- /Since: 0.4.0/
+readDecimal_ :: (Integral a) => ByteString -> a
 {-# SPECIALIZE readDecimal_ ::
     ByteString -> Int,
     ByteString -> Int8,
@@ -244,7 +249,7 @@ readDecimal_ = start
     {-# INLINE isDecimal #-}
     isDecimal w = 0x39 >= w && w >= 0x30
 
-    toDigit :: Integral a => Word8 -> a
+    toDigit :: (Integral a) => Word8 -> a
     {-# INLINE toDigit #-}
     toDigit w = fromIntegral (w - 0x30)
 
@@ -259,7 +264,7 @@ readDecimal_ = start
             w | isDecimal w -> loop0 (toDigit w) (BSU.unsafeTail xs)
               | otherwise   -> 0
 
-    loop0 :: Integral a => a -> ByteString -> a
+    loop0 :: (Integral a) => a -> ByteString -> a
     loop0 m xs
         | m `seq` xs `seq` False = undefined
         | BS.null xs = m
@@ -269,7 +274,7 @@ readDecimal_ = start
               | otherwise   -> m
 
     loop1, loop2, loop3, loop4, loop5, loop6, loop7, loop8
-        :: Integral a => a -> Int -> ByteString -> a
+        :: (Integral a) => a -> Int -> ByteString -> a
     loop1 m n xs
         | m `seq` n `seq` xs `seq` False = undefined
         | BS.null xs = m*10 + fromIntegral n
@@ -699,125 +704,4 @@ _asOctal_overflow =
 -- -}
 
 ----------------------------------------------------------------
-----------------------------------------------------------------
------ Integral logarithms
-
--- TODO: cf. integer-gmp:GHC.Integer.Logarithms made available in version 0.3.0.0 (ships with GHC 7.2.1).
--- <http://haskell.org/ghc/docs/7.2.1/html/libraries/integer-gmp-0.3.0.0/GHC-Integer-Logarithms.html>
-
-
--- This implementation is derived from
--- <http://www.haskell.org/pipermail/haskell-cafe/2009-August/065854.html>
--- modified to use 'quot' instead of 'div', to ensure strictness,
--- and using more guard notation (but this last one's compiled
--- away). See @./test/bench/BenchNumDigits.hs@ for other implementation
--- choices.
---
--- | @numDigits b n@ computes the number of base-@b@ digits required
--- to represent the number @n@. N.B., this implementation is unsafe
--- and will throw errors if the base is @(<= 1)@, or if the number
--- is negative. If the base happens to be a power of 2, then see
--- 'numTwoPowerDigits' for a more efficient implementation.
---
--- We must be careful about the input types here. When using small
--- unsigned types or very large values, the repeated squaring can
--- overflow causing the function to loop. (E.g., the fourth squaring
--- of 10 overflows 32-bits (==1874919424) which is greater than the
--- third squaring. For 64-bit, the 5th squaring overflows, but it's
--- negative so will be caught.) Forcing the type to Integer ensures
--- correct behavior, but makes it substantially slower.
-
-numDigits :: Integer -> Integer -> Int
-{-# INLINE numDigits #-}
-numDigits b0 n0
-    | b0 <= 1   = error (_numDigits ++ _nonpositiveBase)
-    | n0 <  0   = error (_numDigits ++ _negativeNumber)
-    -- BUG: need to check n0 to be sure we won't overflow Int
-    | otherwise = 1 + fst (ilog b0 n0)
-    where
-    ilog b n
-        | n < b     = (0, n)
-        | r < b     = ((,) $! 2*e) r
-        | otherwise = ((,) $! 2*e+1) $! (r `quot` b)
-        where
-        (e, r) = ilog (b*b) n
-
-
--- | Compute the number of base-@2^p@ digits required to represent a
--- number @n@. N.B., this implementation is unsafe and will throw
--- errors if the base power is non-positive, or if the number is
--- negative. For bases which are not a power of 2, see 'numDigits'
--- for a more general implementation.
-numTwoPowerDigits :: (Integral a, Bits a) => Int -> a -> Int
-{-# INLINE numTwoPowerDigits #-}
-numTwoPowerDigits p n0
-    | p  <= 0   = error (_numTwoPowerDigits ++ _nonpositiveBase)
-    | n0 <  0   = error (_numTwoPowerDigits ++ _negativeNumber)
-    | n0 == 0   = 1
-    -- BUG: need to check n0 to be sure we won't overflow Int
-    | otherwise = go 0 n0
-    where
-    go d n
-        | d `seq` n `seq` False = undefined
-        | n > 0     = go (d+1) (n `shiftR` p)
-        | otherwise = d
-
-
--- This implementation is from:
--- <http://www.serpentine.com/blog/2013/03/20/whats-good-for-c-is-good-for-haskell/>
---
--- | Compute the number of base-@10@ digits required to represent
--- a number @n@. N.B., this implementation is unsafe and will throw
--- errors if the number is negative.
-numDecimalDigits :: (Integral a) => a -> Int
-{-# INLINE numDecimalDigits #-}
-numDecimalDigits n0
-    | n0 < 0     = error (_numDecimalDigits ++ _negativeNumber)
-    -- Unfortunately this causes significant (1.2x) slowdown since
-    -- GHC can't see it will always fail for types other than Integer...
-    | n0 > limit = numDigits 10 (toInteger n0)
-    | otherwise  = go 1 (fromIntegral n0 :: Word64)
-    where
-    limit = fromIntegral (maxBound :: Word64)
-    
-    fin n bound = if n >= bound then 1 else 0
-    go k n
-        | k `seq` False = undefined -- For strictness analysis
-        | n < 10        = k
-        | n < 100       = k + 1
-        | n < 1000      = k + 2
-        | n < 1000000000000 =
-            k + if n < 100000000
-                then if n < 1000000
-                    then if n < 10000
-                        then 3
-                        else 4 + fin n 100000
-                    else 6 + fin n 10000000
-                else if n < 10000000000
-                    then 8 + fin n 1000000000
-                    else 10 + fin n 100000000000
-        | otherwise = go (k + 12) (n `quot` 1000000000000)
-
-
-_numDigits :: String
-_numDigits = "numDigits"
-{-# NOINLINE _numDigits #-}
-
-_numTwoPowerDigits :: String
-_numTwoPowerDigits = "numTwoPowerDigits"
-{-# NOINLINE _numTwoPowerDigits #-}
-
-_numDecimalDigits :: String
-_numDecimalDigits = "numDecimalDigits"
-{-# NOINLINE _numDecimalDigits #-}
-
-_nonpositiveBase :: String
-_nonpositiveBase = ": base must be greater than one"
-{-# NOINLINE _nonpositiveBase #-}
-
-_negativeNumber :: String
-_negativeNumber = ": number must be non-negative"
-{-# NOINLINE _negativeNumber #-}
-
-----------------------------------------------------------------
------------------------------------------------------------ fin.
+----------------------------------------------------------- fin.
\ No newline at end of file
